/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003 by Michael Buesch                                  *
 *   email: fsdeveloper@yahoo.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef FILTERPW_H
#define FILTERPW_H

#include <string>

using std::string;

/** struct for containing the filter-options */
typedef struct{
  /* enable or disable filter */
  bool enableFilter;
  /** prevent Sequences of the same char. example: if set to 3, 'aaa' will not be accepted, but 'aa' will */
  unsigned int prevCharSeq;
  /** same as prevCharSeq, but not limited to one char, but the whole charset. example: if 3, 'aow' will be prevented */
  unsigned int prevCharsetSeq;
} filterOpt;

/** character-array
  * Used for generating in GenPw
  * and used for filtering in FilterPw.
  * the charsets have to be ordered (not randomized), for example alphabetical
  */
typedef struct {
  const char *ch_lower;
  const char *ch_upper;
  const char *ch_num;
  const char *ch_special;
  const char *ch_space;
  string ch_additional;// additional chars by user (not hardcoded)
  
  /* NOTE: If a new set is added to the charset, the function whichCs() has to be updated! */
} charArray;

/**
 * filters the "currently generating password" on the fly and scans for "bad" sequences like 'aaa'.
 * Michael Buesch
 **/
class FilterPw
{
public:
  FilterPw();
  FilterPw(const filterOpt *options, const charArray *charset);
  ~FilterPw();
  
  /** set the filter-options (filter-rules) */
  void setOpt(const filterOpt *options);
  /** set charset */
  void setCharset(const charArray *charset);
  /** enable/disable the filter */
  void enableFilter(bool enable = true) { opt.enableFilter = enable; }
  /** scan the new char against the already existing, but not yet complete password-string (tempPw)
    * return: 0  -> success. This char is ok for adding
    *         1  -> failed. prevCharSeq
    *         2  -> failed. prevCharsetSeq
    *         3  -> failed. prevEnumSeq
    */
  int scanNewChar(const string *tempPw, const char newChar);

protected:
  /** current options (filter-rules) */
  filterOpt opt;
  /** charset with which the pw is generated. This will be used as a base for filtering */
  const charArray *chrset;

protected:
  /** find out the charset, the char belongs to. returns the members of the charset */
  bool whichCs(const char ch, string *retcs);
  /** used by whichCs() and isSameCs() to determine if ch is member of cs */
  bool loopThroughCs(const string *cs, const char ch);
  /** test if the two chars are in the same charset */
  bool isInSameCs(const char ch1, const char ch2);

protected:
  /* implementations of the filters. You have to pass the new char and
   * the tempoary password to the filters and it will return true, if
   * it's ok to add the char, but false if the filter doen't let it through.
   */
  bool filter_CharSeq(const string *tempPw, const char newChar);
  bool filter_CharsetSeq(const string *tempPw, const char newChar);
};

#endif
