/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003 by Michael Buesch                                  *
 *   email: fsdeveloper@yahoo.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include <kaboutapplication.h>
#include <klocale.h>

#include <qspinbox.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qprogressbar.h>
#include <qtimer.h>
#include <qfileinfo.h>

#include <kiconloader.h>
#include <kwin.h>
#include <kled.h>
#include <kstandarddirs.h>

#include "pwg.h"
#include "optionswndimpl.h"
#include "selectfilterswndimpl.h"
#include "outputwndimpl.h"
#include "pwgexception.h"


PwG::PwG(QWidget *parent, const char *name) : mainwnd(parent, name)
{
  // open config-file
  confFile = new KConfig(PWG_CONFIG_FILE, false, false);
  confFile->setGroup("GLOBAL");
  if(confFile->readBoolEntry("alwaysOnTop", false))
    KWin::setState(winId(), NET::StaysOnTop);
  else
    KWin::clearState(winId(), NET::StaysOnTop);

  // initialize UI
  setFixedSize(size());
  setCaption(caption() + QString(" - ") + i18n("version") + QString(" ") + QString(VERSION));
  // init tray-icon
  tray = new PwGtray(this);
  KIconLoader icons("pwg");
  tray->setPixmap(icons.loadIcon("pwg", KIcon::Small));
  if(confFile->readBoolEntry("showInTray", false))
    tray->show();

  // initialize class-members
  generate = new GenPw(confFile, this);
  connect(generate, SIGNAL(newPw_signal(string)), this, SLOT(newPwAvailable_slot(string)));
  connect(generate, SIGNAL(pushProgressBar_signal(int, int)), this, SLOT(pushProgressBar_slot(int, int)));
  connect(generate, SIGNAL(closedByUser_signal()), this, SLOT(enableInterface()));
  connect(generate, SIGNAL(rejByFilter_signal(char)), this, SLOT(handleRejectedChar_slot(char)));
  filterLedBlinking = false;
}

PwG::~PwG()
{
  delete generate;
  delete tray;
  delete confFile;
}

void PwG::aboutButton_slot()
{
  KAboutApplication aboutDlg(this);
  aboutDlg.exec();
}

void PwG::quitButton_slot()
{
  close();
}

void PwG::genNowButton_slot()
{
  if(!checkFilterCollision()){
    QMessageBox::critical(this, i18n("Filter collision!"),
      i18n("Sorry, the password can not become generated, because your charset selection\n"
      "collides with your filter selection!\n"
      "Please re-check your selections. Try to select less filters or more charsets."),
      QMessageBox::Ok, 0, 0);
    return;
  }
  enableInterface(false);

  pwRules rules;
  rules.length = pwLenSpinBox->value();
  rules.useLower = useLowerCheckBox->isChecked();
  rules.useNum = useNumCheckBox->isChecked();
  rules.useSpace = useSpaceCheckBox->isChecked();
  rules.useSpecial = useSpecialCheckBox->isChecked();
  rules.useUpper = useUpperCheckBox->isChecked();
  rules.additChars = useAddCheckBox->isChecked() ? additionalCharLineEdit->text().latin1() : "";
  
  filterOpt filterOptions;
  confFile->setGroup("FILTER");
  if(confFile->readNumEntry("charSeq", 2) > 1 ||
   confFile->readNumEntry("charsetSeq", 3) > 1){
    filterOptions.prevCharSeq = confFile->readNumEntry("charSeq", 2);
    filterOptions.prevCharsetSeq = confFile->readNumEntry("charsetSeq", 3);
    filterOptions.enableFilter = true;
  }
  else
    filterOptions.enableFilter = false;

  generate->setRules(&rules);
  generate->setFilterOpt(&filterOptions);
  generate->genNow();
}

void PwG::newPwAvailable_slot(string pw)
{
  OutputWndImpl *out = new OutputWndImpl(this);
  out->setOutPw(pw.c_str());
  out->show();
  enableInterface();
}

void PwG::charsetChanged_slot()
{
  if(useLowerCheckBox->isChecked() ||
     useNumCheckBox->isChecked() ||
     useSpecialCheckBox->isChecked() ||
     useUpperCheckBox->isChecked())
    genNowButton->setEnabled(true);
  else
    genNowButton->setEnabled(false);
  /* note: don't enable if only the "space"-checkbox ist checked,
   *       because this will generate a password containing only spaces.
   */
}

void PwG::optionsButton_slot()
{
  OptionsWndImpl opt(this);
  confFile->setGroup("GLOBAL");
  opt.onTopCheckBox->setChecked(confFile->readBoolEntry("alwaysOnTop", false));
  opt.showTrayCheckBox->setChecked(confFile->readBoolEntry("showInTray", false));
  QString bgpic(confFile->readEntry("rndBackPix"));
  QFileInfo fi(bgpic);
  if(!fi.isReadable())
    bgpic = ::locate("data", PWG_DEFAULT_BG_PIC);
  opt.randBackgrLineEdit->setText(bgpic);

  if(!opt.exec()){
    // write options to config-file
    confFile->writeEntry("alwaysOnTop", opt.onTopCheckBox->isChecked());
    confFile->writeEntry("showInTray", opt.showTrayCheckBox->isChecked());
    confFile->writeEntry("rndBackPix", opt.randBackgrLineEdit->text());
    
    // handle options
    if(opt.onTopCheckBox->isChecked())
      KWin::setState(winId(), NET::StaysOnTop);
    else
      KWin::clearState(winId(), NET::StaysOnTop);

    if(opt.showTrayCheckBox->isChecked())
      tray->show();
    else
      tray->hide();
    generate->randChangeBg(opt.randBackgrLineEdit->text());
  }
}

void PwG::enableInterface(bool enable)
{
  useLowerCheckBox->setEnabled(enable);
  useUpperCheckBox->setEnabled(enable);
  useNumCheckBox->setEnabled(enable);
  useSpecialCheckBox->setEnabled(enable);
  useSpaceCheckBox->setEnabled(enable);
  useAddCheckBox->setEnabled(enable);
  additionalCharLineEdit->setEnabled(enable ? useAddCheckBox->isChecked() : false);
  pwLenSpinBox->setEnabled(enable);
  optionsButton->setEnabled(enable);
  selectFilterButton->setEnabled(enable);

  progressBar->setProgress(0);

  if(enable)
    charsetChanged_slot();
  else
    genNowButton->setEnabled(false);
}

void PwG::pushProgressBar_slot(int totalSteps, int progress)
{
  Q_ASSERT(progress <= totalSteps);
  progressBar->setTotalSteps(totalSteps);
  progressBar->setProgress(progress);
}

void PwG::selectFilterButton_slot()
{
  SelectFiltersWndImpl fil;
  confFile->setGroup("FILTER");
  if(confFile->readNumEntry("charSeq", 2) > 1){
    fil.charSeqCheckBox->setChecked(true);
    fil.charSeqSpinBox->setValue(confFile->readNumEntry("charSeq", 2));
  }
  if(confFile->readNumEntry("charsetSeq", 3) > 1){
    fil.charsetSeqCheckBox->setChecked(true);
    fil.charsetSeqSpinBox->setValue(confFile->readNumEntry("charsetSeq", 3));
  }
  if(!fil.exec()){
    confFile->writeEntry("charSeq", fil.charSeqCheckBox->isChecked() ? fil.charSeqSpinBox->value() : 0);
    confFile->writeEntry("charsetSeq", fil.charsetSeqCheckBox->isChecked() ? fil.charsetSeqSpinBox->value() : 0);
  }
}

void PwG::handleRejectedChar_slot(char)
{
  blinkFilterLed_slot();
}

void PwG::blinkFilterLed_slot()
{
  // note: off -> red; on -> green (but colors may change from system to system)
  if(filterLedBlinking)
    return;
  filterLedBlinking = true;
  filterStatusLed->off();
  QTimer::singleShot(150, this, SLOT(timeoutFilterLed_slot()));
}

void PwG::timeoutFilterLed_slot()
{
  filterStatusLed->on();
  filterLedBlinking = false;
}

void PwG::genHowtoButton_slot()
{
  QMessageBox *mb = new QMessageBox(i18n("HOWTO generate a password"),
    i18n("First you must select one or more charsets in the \"Password-options\" section\n"
    "and the length of the password to become generated. You may also add your own additional\n"
    "characters to become *possibly* included in the password.\n"
    "Now you may want to select one or more filters by pressing the \"Select filters\" button.\n"
    "You can get more information on filters in the select-filters-dialog.\n"
    "But attention: If you select too strict filter-options, you may not be able to generate\n"
    "a password, or it may take very long!\n"
    "After this is done, just click the \"Generate password now\" button. Now a window called\n"
    "the \"Randomizer\" will appear. This window ist used to generate *real* random numbers,\n"
    "from which the password is generated. Now you only have to move your mouse in this\n"
    "randomizer-window. Have a look at the progress-bar in main-window to see the progress of the\n"
    "generation of the password. The randomizer will automatically disappear, once your password\n"
    "is complete."),
    QMessageBox::Information, QMessageBox::Ok, 0, 0, this, 0, FALSE);
  mb->show();
}

bool PwG::checkFilterCollision()
{
  confFile->setGroup("FILTER");
  if(confFile->readNumEntry("charsetSeq") > 1){
    int charsetNum = 0;
    if(useLowerCheckBox->isChecked())
      ++charsetNum;
    if(useNumCheckBox->isChecked())
      ++charsetNum;
    if(useSpecialCheckBox->isChecked())
      ++charsetNum;
    if(useUpperCheckBox->isChecked())
      ++charsetNum;

    if(charsetNum < 2)
      return false;
  }
  return true;
}

void PwG::trayUndockButton_slot()
{
  if(!isVisible())
    show();
  tray->hide();
  confFile->setGroup("GLOBAL");
  confFile->writeEntry("showInTray", false);
}
